import { Wrapper } from './wrapper'
import { Environment, Errors, Events, Utils } from './utils'
import type { RPCState } from './utils'

class Client extends Wrapper {
    private _browser: any = null

    constructor() {
        super()
    }

    set browser(browser: any) {
        this._browser = browser
    }

    public resolveEmitDestination(dataRaw: string) {
        if (!dataRaw) throw new Error(Errors.NO_DATA)

        const state = Utils.prepareExecution(dataRaw)

        switch (state.calledTo) {
            case Environment.SERVER:
                this.emitServer(dataRaw)
                break

            case Environment.BROWSER:
                this.emitBrowser(dataRaw, state)
                break

            case Environment.CLIENT:
                this.emit(state)
                break

            default:
                this.triggerError_(state, Errors.UNKNOWN_ENVIRONMENT)
                break
        }
    }

    private emit(state: RPCState) {
        state = this.verifyEvent_(state)
        if (state.knownError) {
            this.triggerError_(state, state.knownError)
            return
        }

        this.state_[state.eventName](
            ...(Array.isArray(state.data) ? state.data : []),
        )
    }

    private emitServer(dataRaw: string) {
        mp.events.callRemote(Events.SERVER_EVENT_LISTENER, dataRaw)
    }

    private emitBrowser(dataRaw: string, state: RPCState) {
        if (!this._browser) {
            this.triggerError_(state, Errors.NO_BROWSER)
            return
        }

        this._browser.call(Events.LOCAL_EVENT_LISTENER, dataRaw)
    }
}

const client = new Client()
export { client }
