import { Wrapper } from './wrapper'
import { Environment, Events, Utils, type PlayerMp } from './utils'

class Server extends Wrapper {
    constructor() {
        super()

        mp.events.add(
            Events.SERVER_EVENT_LISTENER,
            async (player: PlayerMp, dataRaw: string) => {
                this.emit(player, dataRaw)
            },
        )
    }

    public resolveEmitDestination(player: PlayerMp, dataRaw: string) {
        let state = Utils.prepareExecution(dataRaw)

        switch (state.calledTo) {
            case Environment.SERVER:
                this.emit(player, dataRaw)
                break

            default:
                this.emitClient(player as PlayerMp, dataRaw)
                break
        }
    }

    private emitClient(player: PlayerMp, dataRaw: string) {
        player.call(Events.LOCAL_EVENT_LISTENER, [dataRaw])
    }

    private emit(player: PlayerMp, dataRaw: string) {
        let state = Utils.prepareExecution(dataRaw)

        state = this.verifyEvent_(state)
        if (state.knownError) {
            this.triggerError_(state, state.knownError)
            return
        }

        const { eventName, data } = Utils.prepareExecution(dataRaw)
        this.state_[eventName](player, ...data)
    }
}

const server = new Server()
export { server }
